package org.mule.module.datapack;

import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.StringUtils;
import org.mule.api.MuleMessage;
import org.mule.api.transformer.TransformerException;
import org.mule.module.datapack.columns.Column;

/**
 * 
 * @author marianogonzalez
 *
 */
public class BeanOutputTransformer extends DataPackTransformer {

	private String beanClass;
	
	@Override
	protected Object datapack(MuleMessage message, String outputEncoding) throws TransformerException {
		
		Object bean = this.instanciateBean();
		
		try {
			
			for (Column column : this.columns) {
				BeanUtils.setProperty(bean, column.getColumnName(), this.evaluate(message, column));
			}
			
		} catch (IllegalAccessException e) {
			this.handleSetPropertyException(e);
		} catch (TransformerException e) {
			this.handleSetPropertyException(e);
		} catch (InvocationTargetException e) {
			this.handleSetPropertyException(e);
		}
		
		
		return bean;
	}


	private Object instanciateBean() {
		if (StringUtils.isEmpty(this.beanClass)) {
			throw new IllegalStateException("beanClass not specified");
		}
		
		Class<?> clazz = null;
		
		try {
			clazz = Class.forName(this.beanClass);
		} catch (ClassNotFoundException e) {
			throw new IllegalArgumentException(this.beanClass + " not found in classpath");
		}
		
		Object bean = null;
		
		try {
			bean = clazz.newInstance();
		} catch (IllegalAccessException e) {
			this.handleInstanciationException(e);
		} catch (InstantiationException e) {
			this.handleInstanciationException(e);
		}
		return bean;
	}

	
	private void handleSetPropertyException(Throwable e) {
		throw new RuntimeException("Error setting property on bean of class " + this.beanClass, e);
	}
	
	private void handleInstanciationException(Throwable e) {
		throw new RuntimeException("Error instanciating bean", e);
	}


	public String getBeanClass() {
		return beanClass;
	}

	public void setBeanClass(String beanClass) {
		this.beanClass = beanClass;
	}
	
}
