package org.mule.module.datapack;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.mule.api.MuleMessage;
import org.mule.api.transformer.TransformerException;
import org.mule.module.datapack.columns.Column;
import org.mule.module.datapack.i18n.DataPackMessages;

/**
 * Base class for datapack transformers.
 * Basically extends {@link org.mule.module.datapack.ExpressionCapableTransformer}
 * and adds a list of {@link org.mule.module.datapack.columns.Column}
 * plus some basic behaviour
 * 
 * @author marianogonzalez
 */
public abstract class DataPackTransformer extends ExpressionCapableTransformer {

	protected List<Column> columns;
	private String target;
	
    public List<Column> getColumns()
    {
        return columns;
    }

    public void setColumns(List<Column> columns)
    {
        this.columns = columns;
    }
    
    /**
     * Validates that the columns are not null or empty and delegates into {@link org.mule.module.datapack.DataPackTransformer.datapack(MuleMessage, String)}
     * 
     * @see org.mule.module.datapack.DataPackTransformer.datapack(MuleMessage, String)
     * @see org.mule.transformer.AbstractMessageTransformer#transformMessage(org.mule.api.MuleMessage, java.lang.String)
     * @throws org.mule.api.transformer.TransformerException if no columns or if there's an exception during transformation
     */
    @Override
    public Object transformMessage(MuleMessage message, String outputEncoding) throws TransformerException {
    	
    	 if (columns == null || columns.size() == 0)
         {
             throw new TransformerException(DataPackMessages.noColumnsDefinedMessage());
         }
    	 
    	 Object payload = this.datapack(message, outputEncoding);
    	 
    	 if (StringUtils.isEmpty(this.target))
    	 {
    		 return payload;
    	 } else
    	 {
    		 message.setOutboundProperty(this.target, payload);
    		 return message;
    	 }
    }
    
    /**
     * IMplementors will perform the transformation in this method
     * @param message
     * @param outputEncoding
     * @return
     * @throws TransformerException
     */
    protected abstract Object datapack(MuleMessage message, String outputEncoding) throws TransformerException;
    
    /**
     * Utility method to evaluate the column
     * @param message
     * @param column
     * @return
     * @throws TransformerException
     */
    protected String evaluate(MuleMessage message, Column column) throws TransformerException {
    	return column.evaluateColumn(message,  muleContext, expressionManager, patternInfo);
    }

	public String getTarget() {
		return target;
	}

	public void setTarget(String target) {
		this.target = target;
	}
}
