package org.mule.module.datapack;

import org.mule.api.expression.ExpressionManager;
import org.mule.api.lifecycle.InitialisationException;
import org.mule.transformer.AbstractMessageTransformer;
import org.mule.util.TemplateParser;

/**
 * Base class for transformers that need to evaluate MuleExpressions.
 * Provides default access to a {@link org.mule.api.expression.ExpressionManager}
 * and to a {@link org.mule.util.TemplateParser.PatternInfo}
 * 
 * @author marianogonzalez
 * 
 */
public abstract class ExpressionCapableTransformer extends AbstractMessageTransformer {

	/**
	 * Automatically initialized invoking {@link org.mule.util.TemplateParser.getStyle()} 
	 */
	protected final TemplateParser.PatternInfo patternInfo = TemplateParser.createMuleStyleParser().getStyle();

	protected ExpressionManager expressionManager;

	/**
	 * Calls the super method and then initializes {@link org.mule.module.datapack.ExpressionCapableTransformer.expressionManager}
	 * with {@link org.mule.api.MuleContext.getExpressionManager()}
	 */
	@Override
	public void initialise() throws InitialisationException {
		super.initialise();
		expressionManager = muleContext.getExpressionManager();
	}

	public ExpressionManager getExpressionManager() {
		return expressionManager;
	}

	public void setExpressionManager(ExpressionManager expressionManager) {
		this.expressionManager = expressionManager;
	}

	public TemplateParser.PatternInfo getPatternInfo() {
		return patternInfo;
	}
	
}
