package org.mule.module.datapack.exception;


/**
 * 
 * @author mariano.gonzalez@mulesoft.com
 *
 */
public class DataPackException extends RuntimeException {

	private static final long serialVersionUID = 1110570580002399687L;

	private static final String message = "Error evaluating expression in datapack";
	
	private String expression;
	private Object payload;
	private String columnName;
	private String transformerName;
	
	public DataPackException(String expression, Object payload, String columnName) {
		super(message);
		this.expression = expression;
		this.payload = payload;
		this.columnName = columnName;
	}
	
	public DataPackException(String expression, Object payload, String columnName, Throwable cause) {
		super(message, cause);
		this.expression = expression;
		this.payload = payload;
		this.columnName = columnName;
	}

	public String getExpression() {
		return expression;
	}
	
	public Object getPayload() {
		return payload;
	}

	public String getColumnName() {
		return columnName;
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder(this.getMessage())
			.append("\nexpression: ").append(this.expression)
			.append("\npayload: ").append(this.payload)
			.append("\ncolumnName: ").append(this.columnName)
			.append("\ntransformer: ").append(this.transformerName);
		
		if (this.getCause() != null) {
			builder.append("\ncause: ").append(this.getCause().getMessage());
		}
		
		return builder.toString();
	}

	public String getTransformerName() {
		return transformerName;
	}

	public void setTransformerName(String transformerName) {
		this.transformerName = transformerName;
	}
	
	
	
}
